@@ -369,6 +369,7 @@ and venv_globals =
      venv_cache                              : Omake_cache.t;
      mutable venv_files                      : NodeSet.t;
      mutable venv_directories                : venv DirTable.t;
+     mutable venv_excluded_directories       : DirSet.t;
      mutable venv_explicit_rules             : erule list;
      mutable venv_explicit_targets           : NodeSet.t;
      mutable venv_explicit_new               : erule list;
@@ -1631,14 +1632,14 @@ let venv_save_static_values venv =
 (*
  * Convert a filename to a node.
  *)
-let venv_intern venv name =
-   Node.intern venv.venv_inner.venv_mount venv.venv_inner.venv_dir name
+let venv_intern venv phony_flag name =
+   Node.intern venv.venv_inner.venv_mount phony_flag venv.venv_inner.venv_dir name
 
-let venv_intern_cd venv dir name =
-   Node.intern venv.venv_inner.venv_mount dir name
+let venv_intern_cd venv phony_flag dir name =
+   Node.intern venv.venv_inner.venv_mount phony_flag dir name
 
 let venv_intern_target venv multiple name =
-   let node = Node.intern venv.venv_inner.venv_mount venv.venv_inner.venv_dir name in
+   let node = Node.intern venv.venv_inner.venv_mount PhonyOK venv.venv_inner.venv_dir name in
       match multiple with
          RuleScannerSingle
        | RuleScannerMultiple ->
@@ -1656,12 +1657,12 @@ let venv_intern_list venv names =
 let node_set_of_list nodes =
    List.fold_left NodeSet.add NodeSet.empty nodes
 
-let node_set_add_names venv nodes names =
+let node_set_add_names venv phony_flag nodes names =
    List.fold_left (fun nodes name ->
-         NodeSet.add nodes (venv_intern venv name)) nodes names
+         NodeSet.add nodes (venv_intern venv phony_flag name)) nodes names
 
-let node_set_of_names venv names =
-   node_set_add_names venv NodeSet.empty names
+let node_set_of_names venv phony_flag names =
+   node_set_add_names venv phony_flag NodeSet.empty names
 
 (*
  * Add a mount point.
@@ -1689,7 +1690,7 @@ let compile_source_core venv s =
    if is_wild s then
       SourceWild (wild_compile s)
    else
-      SourceNode (venv_intern venv s)
+      SourceNode (venv_intern venv PhonyOK s)
 
 let compile_source venv (kind, s) =
    kind, compile_source_core venv s
@@ -1701,7 +1702,7 @@ let subst_source_core venv subst source =
    match source with
       SourceWild wild ->
          let s = wild_subst subst wild in
-            venv_intern venv s
+            venv_intern venv PhonyOK s
     | SourceNode node ->
          node
 
@@ -1712,7 +1713,7 @@ let subst_source venv subst (kind, source) =
  * No wildcard matching.
  *)
 let intern_source venv (kind, source) =
-   Node.escape kind (venv_intern venv source)
+   Node.escape kind (venv_intern venv PhonyOK source)
 
 (*
  * For variables, try to look them up as 0-arity functions first.
@@ -2075,6 +2076,7 @@ let create options dir exec cache =
         venv_cache                      = cache;
         venv_files                      = NodeSet.empty;
         venv_directories                = DirTable.empty;
+        venv_excluded_directories       = DirSet.empty;
         venv_explicit_rules             = [];
         venv_explicit_new               = [];
         venv_explicit_targets           = NodeSet.empty;
@@ -2121,7 +2123,7 @@ let create options dir exec cache =
    let venv = venv_add_phony venv (Lm_location.bogus_loc makeroot_name) [".PHONY"] in
    let venv = venv_add_var_dynamic venv cwd_sym (ValDir cwd) in
    let venv = venv_add_var_dynamic venv stdlib_sym (ValDir Dir.lib) in
-   let venv = venv_add_var_dynamic venv stdroot_sym (ValNode (Node.intern mount Dir.lib "OMakeroot")) in
+   let venv = venv_add_var_dynamic venv stdroot_sym (ValNode (Node.intern mount PhonyProhibited Dir.lib "OMakeroot")) in
    let venv = venv_add_var_dynamic venv ostype_sym (ValString Sys.os_type) in
    let venv = venv_add_wild_match venv wild_string in
    let omakepath =
@@ -2177,6 +2179,7 @@ let venv_get_pervasives venv node =
         venv_cache                      = cache;
         venv_files                      = NodeSet.empty;
         venv_directories                = DirTable.empty;
+        venv_excluded_directories       = DirSet.empty;
         venv_explicit_rules             = [];
         venv_explicit_new               = [];
         venv_explicit_targets           = NodeSet.empty;
@@ -2321,11 +2324,17 @@ let venv_add_dir venv =
       globals.venv_directories <- DirTable.add globals.venv_directories venv.venv_inner.venv_dir venv
 
 let venv_directories venv =
-   venv.venv_inner.venv_globals.venv_directories
+   let globals = venv.venv_inner.venv_globals in
+      DirSet.fold DirTable.remove globals.venv_directories globals.venv_excluded_directories
 
 let venv_add_explicit_dir venv dir =
    let globals = venv.venv_inner.venv_globals in
-      globals.venv_directories <- DirTable.add globals.venv_directories dir venv
+      globals.venv_directories <- DirTable.add globals.venv_directories dir venv;
+      globals.venv_excluded_directories <- DirSet.remove globals.venv_excluded_directories dir
+
+let venv_remove_explicit_dir venv dir =
+   let globals = venv.venv_inner.venv_globals in
+      globals.venv_excluded_directories <- DirSet.add globals.venv_excluded_directories dir
 
 (*
  * When a file is read, remember it as a configuration file.
@@ -2564,7 +2573,7 @@ let venv_explicit_rules venv =
    let info =
       { explicit_targets      = NodeTable.empty;
         explicit_deps         = NodeTable.empty;
-        explicit_directories  = venv.venv_inner.venv_globals.venv_directories;
+        explicit_directories  = venv_directories venv
       }
    in
       venv_explicit_flush venv;
