@@ -47,8 +47,6 @@ open Omake_cache_type
 open Omake_command_type
 open Omake_command_digest
 open Omake_options_type
-open Omake_shell_type
-open Omake_shell_job
 open Omake_symbol
 
 module Pos = MakePos (struct let name = "Omake_eval" end)
@@ -247,11 +245,11 @@ and find_include_file venv pos loc filename =
    let cache = venv_cache venv in
       if not (Filename.is_relative filename) || not (Filename.is_implicit filename) then
          let fullname = filename ^ omake_file_suffix in
-         let node1 = venv_intern venv fullname in
+         let node1 = venv_intern venv PhonyProhibited fullname in
             if Omake_cache.exists cache node1 then
                node1
             else
-               let node2 = venv_intern venv filename in
+               let node2 = venv_intern venv PhonyProhibited filename in
                   if Omake_cache.exists cache node2 then
                      node2
                   else
@@ -652,14 +650,15 @@ and file_of_value venv pos file =
       match file with
          ValNode node ->
             node
-       | ValDir _
+       | ValDir dir ->
+            Node.node_of_dir dir
        | ValData _
        | ValString _
        | ValSequence _
        | ValQuote _
        | ValInt _
        | ValFloat _ ->
-            venv_intern venv (string_of_value venv pos file)
+            venv_intern venv PhonyExplicit (string_of_value venv pos file)
        | ValArray _
        | ValNone
        | ValKey _
@@ -1600,7 +1599,7 @@ let eval_open_file = open_ir
  * Project compiler.
  *)
 let compile venv =
-   let node = venv_intern venv makeroot_name in
+   let node = venv_intern venv PhonyProhibited makeroot_name in
    let venv = venv_add_file venv node in
    let loc = bogus_loc (Node.fullname node) in
    let pos = string_pos "compile" (loc_exp_pos loc) in
