@@ -28,6 +28,7 @@ open Lm_printf
 
 open Omake_ir
 open Omake_env
+open Omake_node_sig
 open Omake_node
 open Omake_lexer
 open Omake_parser
@@ -237,7 +238,7 @@ let node_value_of_value venv pos v =
        | ValInt _
        | ValFloat _ ->
             let name = string_of_value venv pos v in
-            let node = venv_intern venv name in
+            let node = venv_intern venv PhonyExplicit name in
             let cache = venv_cache venv in
                if Omake_cache.is_dir cache node then
                   ValDir (venv_intern_dir venv name)
@@ -371,7 +372,7 @@ let in_channel_of_any_value venv pos v =
                if is_channel_string s then
                   prim_channel_of_string venv pos s, false
                else
-                  let node = venv_intern venv s in
+                  let node = venv_intern venv PhonyProhibited s in
                   let name = Node.fullname node in
                   let fd =
                      try Lm_unix_util.openfile name [Unix.O_RDONLY] 0 with
@@ -420,7 +421,7 @@ let out_channel_of_any_value venv pos v =
                if is_channel_string s then
                   prim_channel_of_string venv pos s, false
                else
-                  let node = venv_intern venv s in
+                  let node = venv_intern venv PhonyProhibited s in
                   let name = Node.fullname node in
                   let fd =
                      try Lm_unix_util.openfile name [Unix.O_WRONLY; Unix.O_CREAT; Unix.O_TRUNC] 0o666 with
