@@ -894,7 +894,7 @@ let listing_find cache listing s =
          DirEntryCore entry ->
             entry
        | LazyEntryCore (dir, s) ->
-            let node = Node.intern no_mount_points dir s in
+            let node = Node.intern no_mount_points PhonyProhibited dir s in
             let entry =
                if is_dir cache node then
                   DirEntry (Dir.chdir dir s)
@@ -977,7 +977,7 @@ let ls_exe_path_unix cache dirs =
 let rec search_exe_win32 cache entries =
    match entries with
       (dir, s) :: entries ->
-         let node = Node.intern no_mount_points dir s in
+         let node = Node.intern no_mount_points PhonyProhibited dir s in
             if is_dir cache node then
                search_exe_win32 cache entries
             else
@@ -988,7 +988,7 @@ let rec search_exe_win32 cache entries =
 let rec search_exe_unix cache entries =
    match entries with
       (dir, s) :: entries ->
-         let node = Node.intern no_mount_points dir s in
+         let node = Node.intern no_mount_points PhonyProhibited dir s in
             if is_exe_file cache node then
                node
             else
@@ -999,7 +999,7 @@ let rec search_exe_unix cache entries =
 let rec search_exe_cygwin cache entries =
    match entries with
       (dir, s) :: entries ->
-         let node = Node.intern no_mount_points dir s in
+         let node = Node.intern no_mount_points PhonyProhibited dir s in
             if Filename.check_suffix s ".exe" || Filename.check_suffix s ".bat" || is_exe_file cache node then
                node
             else
