@@ -8,7 +8,7 @@
  * ----------------------------------------------------------------
  *
  * @begin[license]
- * Copyright (C) 2003 Jason Hickey, Caltech
+ * Copyright (C) 2003-2006 Mojave Group, Caltech
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,8 +24,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * Author: Jason Hickey
- * @email{jyh@cs.caltech.edu}
+ * Author: Jason Hickey <jyh@cs.caltech.edu>
+ * Modified by: Aleksey Nogin <nogin@cs.caltech.edu>
  * @end[license]
  *)
 open Lm_hash
@@ -1053,13 +1053,13 @@ struct
     * Intern a string with no escapes.
     * This version ignores mount points.
     *)
-   let intern_simple dir name =
+   let intern_simple phony_ok dir name =
       let { base_nodes = nodes } = base in
 
       (* Parse the .PHONY syntax *)
       let name = Lm_filename_util.unescape_string name in
-         match parse_phony_name name with
-            PhonyDirString name ->
+         match parse_phony_name name, phony_ok with
+            PhonyDirString name, (PhonyOK | PhonyExplicit) ->
                (* Name is .PHONY/... *)
                let hash, dir, key, name = new_file dir name in
                let node = NodePhonyDir (hash, dir, key, name) in
@@ -1068,7 +1068,7 @@ struct
                          base.base_nodes <- NodeTable.add nodes node node;
                          node)
 
-          | PhonyGlobalString name ->
+          | PhonyGlobalString name, (PhonyOK | PhonyExplicit) ->
                (* Name is /.PHONY/name *)
                let hash = Hash.hash_global_phony name in
                let node = NodePhonyGlobal (hash, name) in
@@ -1077,7 +1077,10 @@ struct
                          base.base_nodes <- NodeTable.add nodes node node;
                          node)
 
-          | PhonySimpleString ->
+          | (PhonyDirString _| PhonyGlobalString _), PhonyProhibited ->
+               raise (Invalid_argument "Omake_node.Node.intern: NodePhony is not allowed");
+
+          | PhonySimpleString, PhonyOK ->
                (* Name is simple, without path separators *)
                let key  = Lm_filename_util.normalize_string name in
                let hash = Hash.hash_file dir key in
@@ -1096,7 +1099,8 @@ struct
                                            base.base_nodes <- NodeTable.add nodes node node;
                                            node)
 
-          | PhonyPathString ->
+          | PhonySimpleString, (PhonyExplicit | PhonyProhibited)
+          | PhonyPathString, _ ->
                (* Otherwise, it is a normal file *)
                let hash, dir, key, name = new_file dir name in
                let node = NodeFile (hash, dir, key, name) in
@@ -1385,7 +1389,7 @@ struct
       else
          symlink_file_unix
 
-   let intern mount dir name =
+   let intern mount phony_ok dir name =
       let { mount_info = mount_info;
             mount_points = mounts
           } = mount
@@ -1394,7 +1398,7 @@ struct
             mount_file_reset = reset_file
           } = mount_info
       in
-      let node = intern_simple dir name in
+      let node = intern_simple phony_ok dir name in
       let rec search mounts =
          match mounts with
             (dir_dst, dir_src, options) :: mounts ->
