@@ -4,7 +4,7 @@
  * ----------------------------------------------------------------
  *
  * @begin[license]
- * Copyright (C) 2004-2005 Mojave Group, Caltech
+ * Copyright (C) 2004-2006 Mojave Group, Caltech
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,8 +20,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * Author: Jason Hickey
- * @email{jyh@cs.caltech.edu}
+ * Author: Jason Hickey @email{jyh@cs.caltech.edu}
+ * Modified by: Aleksey Nogin @email{nogin@cs.caltech.edu}
  * @end[license]
  *)
 open Lm_printf
@@ -30,8 +30,10 @@ open Lm_location
 
 open Omake_ir
 open Omake_env
+open Omake_node_sig
 open Omake_node
 open Omake_symbol
+open Omake_value
 open Omake_shell_sys
 open Omake_shell_type
 open Omake_shell_sys_type
@@ -576,6 +578,41 @@ let create_apply venv pgrp bg stdin stdout stderr apply =
             cleanup ();
             raise exn
 
+(*
+ * Resolve the absolute name of the executable.
+ *)
+let find_executable venv pos loc exe =
+   let pos = string_pos "find_executable" pos in
+   let cache = venv_cache venv in
+   let node =
+      if not (Filename.is_relative exe) || Lm_string_util.contains_any exe Lm_filename_util.separators then
+         let node = venv_intern venv PhonyProhibited exe in
+            if Omake_cache.exists cache node then
+               node
+            else if (Sys.os_type = "Win32" || Sys.os_type = "Cygwin")
+               && not (Filename.check_suffix exe ".exe" || Filename.check_suffix exe ".bat")
+            then
+               let node = venv_intern venv PhonyProhibited (exe ^ ".exe") in
+                  if Omake_cache.exists cache node then
+                     node
+                  else
+                     let node = venv_intern venv PhonyProhibited (exe ^ ".bat") in
+                        if Omake_cache.exists cache node then
+                           node
+                        else
+                           raise (OmakeException (loc_pos loc pos, StringStringError ("command not found", exe)))
+            else
+               raise (OmakeException (loc_pos loc pos, StringStringError ("command not found", exe)))
+      else
+         let path = venv_find_var venv ScopeGlobal pos loc path_sym in
+         let path = List.map (dir_of_value venv pos) (values_of_value venv pos path) in
+         let path = Omake_cache.ls_exe_path cache path in
+            try Omake_cache.exe_find cache path exe with
+               Not_found ->
+                  raise (OmakeException (loc_pos loc pos, StringStringError ("command not found", exe)))
+   in
+      Node.absname node
+
 (*
  * Start a command.
  *)
@@ -590,6 +627,8 @@ let create_command venv pgrp bg stdin stdout stderr command =
          cmd_stderr = stderr_divert
        } = command
    in
+   let pos = string_pos "Omake_shell_job.create_command" (loc_exp_pos loc) in
+   let exe = find_executable venv pos loc exe in
    let stdin, close_stdin, stdout, close_stdout, stderr =
       create_channels stdin stdin_file append stdout stdout_file stderr_divert stderr
    in
